/*
 * Oleg Moskalenko, 2013
 */

#include "udpbug.h"

static int set_socket_nonblocking(int fd) {

    int flags;
    
    /* Posix */
    if (-1 == (flags = fcntl(fd, F_GETFL, 0)))
      flags = 0;
    return fcntl(fd, F_SETFL, flags | O_NONBLOCK);
}     

int main()
{
	ioa_addr server_addr, client_addr, remote_addr;

	init_addrs(&server_addr,&client_addr);

	/* Step 1: */
	int server_fd = open_socket(); /* SOCKET A */

	/* Step 2: */
	addr_bind(server_fd, &server_addr);

	/* Step 3: */
	char s[2048];
	int plen = udp_recvfrom(server_fd, &remote_addr, s, (int)sizeof(s));

	/* Step 7: */
	if(plen<0) {
		fprintf(stderr,"Fatal error: cannot receive packet 1.\n");
		exit(-1);
	}
	if(!addr_eq(&client_addr,&remote_addr)) {
		fprintf(stderr,"Fatal error: Packet 1 received from unknown source.\n");
		exit(-1);
	}
	s[plen]=0;
	if(strcmp(s,PACKET1)) {
		fprintf(stderr,"Fatal error: Wrong packet 1 received.\n");
		exit(-1);
	}

	/* Step 8: */
	int client_fd = open_socket(); /* SOCKET B */

	/* Step 9: */
	addr_bind(client_fd, &server_addr);

	/* Step 10: */
	addr_connect(client_fd, &client_addr);

	sleep(1);

	/* Step 11: */
	printf("Step 11 start\n");
	udp_send(client_fd, NULL, PACKET2, (int)strlen(PACKET2));
	printf("Step 11 end\n");

	set_socket_nonblocking(client_fd);
	set_socket_nonblocking(server_fd);

	printf("Step 14 start\n");

	unsigned int cycle = 0;
	while(cycle++<2000000) {
	  /* Step 14: */
	  plen = udp_recvfrom(client_fd, &remote_addr, s, (int)sizeof(s));
	  if(plen<0) {
	    plen = udp_recvfrom(server_fd, &remote_addr, s, (int)sizeof(s));
	    if(plen>0) {
	      s[plen]=0;
	      if(!strcmp(s,PACKET3)) {
		fprintf(stderr,"ERROR: I RECEIVED PACKET 3 ON SOCKET A\n");
		exit(-1);
	      } else {
		fprintf(stderr,"ERROR: I received unknown packet on socket A\n");
		exit(-1);
	      }
	    }
	  } else {
	    if(!strcmp(s,PACKET3)) {
	      fprintf(stderr,"SUCCESS: I RECEIVED PACKET 3 ON SOCKET B\n");
	      break;
	    } else {
	      fprintf(stderr,"ERROR: I received unknown packet on socket B\n");
	      exit(-1);
	    }
	  }
	}

	printf("Step 14 end\n");

	return 0;
}
